/* -*- linux-c -*- */
/*****************************************************************************
 * atob.c: a simple ascii to binary converter                                *
 *                                                                           *
 * heavily modifed from hdump by Mike Haaland <mikeh@hypertech.com>          *
 *                                                                           *
 * author: Sean Dwyer <ewe2@can.org.au>                                      *
 *                                                                           *
 * Why do this? Because you may not have access to a screen hex editor!      *
 * (think of disk crashes, shell loss, etc).                                 *
 *                                                                           *
 * Usage: atob < textfile > dumpfile                                         *
 * where textfile is a bunch of lines up to 64 characters long - the program *
 * will ignore lines longer than that. You could alter that if you really    *
 * wished to, but od et. al. sees it pretty much that way, so its easiest.   *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

/* ansi decls */
int nyb(int);
int hex(char *);
int convert(char *);

unsigned char tmpbuf[16]; /* hex buffer for conversion */
char line[80];            /* line to scan */

/*****************************************************************************
 * int nyb(int) - takes the index in a string and de-ascifies it             *
 * returns the shifted character. Checks for non-hex values.                 *
 * files will still be created without the bad characters (beware!)          *
 *****************************************************************************/

int nyb(int c)
{
    /* control chars and uneven bytes will be caught here */
    if (c < '0' || (c > '9' && c < 'a') || c > 'f') {
        fprintf(stderr,"atob: bad hex value\n");
        exit(1);
    }
    
    if(c > '9')
        return(c - 'a' + 10);
    else
        return(c - '0');
}

/*****************************************************************************
 * int hex(char *) - takes the result of nyb() shifted and added to itself   *
 * to return a hexadecimal byte.                                             *
 *****************************************************************************/


int hex(char *s)
{
    return(nyb(*s) << 4 | nyb(*(s+1)));
}

/*****************************************************************************
 * int convert(char *) - for each line, converts a character into its        *
 * hexadecimal equivalent unless the line ends.                              *
 * returns the result.                                                       *
 *****************************************************************************/

int convert(char *s)
{
    int n;

    n = 0;
    while(n<32) {
        if((*s == ' ') || (*s == 0x0D) || (*s == 0x0A))
            break;
        tmpbuf[n] = hex(s);
        s += 2;
        n++;
    }
    return(n);
}

/*****************************************************************************
 * main processing;                                                          *
 *                                                                           *
 * open stdout , get a line, convert it, write out the line. convert() does  *
 * all the hard work, then output on stdout.                                 *
 *                                                                           *
 *****************************************************************************/

int main()
{
    int c;

    while((fgets(line,sizeof(line),stdin)) !=NULL) {
        c = convert(line);
        fwrite(tmpbuf,c,1,stdout);
    }
    return(0);
}
