/*****************************************************************************
 * btoa1.c - read a binary file and output it in terse ASCII format          *
 *                                                                           *
 * Heavily modified from undump by Mike Haaland <mikeh@hypercom.com>         *
 *                                                                           *
 * author: Sean Dwyer <ewe2@cvis.com.au>                                     *
 *                                                                           *
 * this program is the kinder, file name specified version.                  *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

/* ansi decls */
int usage(void);
char *ohex(char *,int); 

/* global vars */
char *fmt = "%s\n"; /* the output string */
int maxwide = 16;   /* maximum ascii characters per buffer (half the actual
                     * number to allow proper hex conversion)
                     */
int maxline = 16;   /* maximum line length */
char *prog;         /* proggie name for errors */

/*--------------------------------------------------------------------------*\
 * usage() - explain the program if no args given.                          *
\*--------------------------------------------------------------------------*/

int usage(void)
{
   fprintf(stderr,"%s - binary to ascii converter\n",prog);
   fprintf(stderr,"   usage: btoa dumpfile texfile\n");
   exit(1);
}

/*--------------------------------------------------------------------------*\
 * char *ohex(char *,int) - modifies an ASCII string to return a hex string *
 * and halves the size of the string in the process                         *
\*--------------------------------------------------------------------------*/


char *ohex(char *s, int ch)
{
    char tstr[10];    /* hold 8 + 2 bits */

	sprintf(tstr,"%02x",ch);
	*s++ = tstr[0];
	*s++ = tstr[1];  /* grab the hex and truncate the string */
	*s = '\0';
	return(s);
}

/*--------------------------------------------------------------------------*\
 * main() - takes input on stdin, converts the stream to hex and prints out *
 * ASCII lines of hex characters.                                           *
\*--------------------------------------------------------------------------*/


int main(int argc, char *argv[])
{
    FILE *in,*out; /* input and output files */
    int ch;        /* current byte */
    int i = 0;     /* max character buffer counter */
    int k = 0;     /* max line counter */

    char buf[80];   /* buffer to stuff results in */
    char *s;        /* pointer to buffer */
    prog = argv[0]; /* save program name for errors */
    
    s = buf;        /* init buffer */
    *buf = '\0';

    /* process args */
    if (argc < 3)
        usage();

    if ((in = fopen(argv[1],"rb")) == NULL) {
        usage();
    }

    if ((out = fopen(argv[2],"w")) == NULL) {
        usage();
    }

    while((ch = fgetc(in)) !=EOF) {
        s = ohex(s,ch);             /* convert bytes to ascii */
        i++;                        /* count 'em */
        if (i == maxwide)           /* buffer is full */
        {
            fprintf(out,fmt,buf);
            k++;                    /* keep track of the line */
            i = 0;                  /* reset buffer and count */
            s = buf;
            *buf = '\0';
            if (k == maxline)       /* reset the line if full */
                    k = 0;
        }
    }
    if (i > 0)                      /* print whatever's left */
        fprintf(out,fmt,buf);

    fclose(in);                     /* close streams */
    fclose(out);
    return (0);
}
